def csv_to_list(value):
    """
    Converts the given value to a list of strings, spliting by ',' and removing empty strings.
    :param str value: the value to split to strings.
    :return: a list of strings.
    :rtype: List[str]
    """
    return list(filter(None, [x.strip() for x in value.split(',')]))



from __future__ import print_function, absolute_import

import logging
from six import iteritems

from flask import request_started, request_tearing_down, got_request_exception
from flask import current_app, session, request as _request
from honeybadger import payload
from honeybadger.utils import filter_dict

from .base import HoneybadgerExtension
from ._helpers import csv_to_list

DEFAULT_SKIP_HEADERS = ', '.join([
    'Authorization',
    'Proxy-Authorization'
])

logger = logging.getLogger(__name__)


class HoneybadgerFlask(HoneybadgerExtension):
    """
    Flask extension for honeybadger. Initializes honeybadger and adds a flask error handler that notifies honeybadger
    of exceptions.
    """
    def __init__(self, app=None, context_generators={}, report_exceptions=False):
        """
        Initialize Honeybadger.
        :param flask.Application app: the application to wrap for the exception.
        :param dict context_generators: a dictionary with key the name of additional context property to add and value
        a callable that generates the actual value of the property.
        :param bool report_exceptions: whether to automatically report exceptions on requests or not.
        """
        super(HoneybadgerFlask, self).__init__(context_generators=context_generators,
                                               report_exceptions=report_exceptions)
        self.app = app
        self.skip_headers = []
        if app is not None:
            self.init_app(app, context_generators=context_generators, report_exceptions=report_exceptions)

    def init_app(self, app, context_generators={}, report_exceptions=False):
        """
        Initialize honeybadger and listen for errors
        :param app: the Flask application object.
        :param context_generators: a dictionary with key the name of additional context property to add and value a
        callable that generates the actual value of the property.
        :param bool report_exceptions: whether to automatically report exceptions on requests or not.
        """
        self.context_generators = context_generators
        self.report_exceptions = report_exceptions
        self.initialize_honeybadger(app.config)
        self._patch_generic_request_payload()
        self.skip_headers = set(csv_to_list(app.config.get('HONEYBADGER_EXCLUDE_HEADERS', DEFAULT_SKIP_HEADERS)))
        request_started.connect(self.setup_context, sender=app, weak=False)
        request_tearing_down.connect(self.reset_context, sender=app, weak=False)
        logger.info('Honeybadger Flask helper installed')

        if self.report_exceptions:
            logger.info('Enabling auto-reporting exceptions')
            got_request_exception.connect(self._handle_exception, sender=app, weak=False)

    def _patch_generic_request_payload(self):
        """
        Monkey-patches Honeybadger's generic_request_payload to add information from Flask.
        """
        def generic_request_payload_decorator(original):
            def _wrapper(request, context, config):
                current_view = current_app.view_functions[_request.endpoint]
                if hasattr(current_view, 'view_class'):
                    component = '.'.join((current_view.__module__, current_view.view_class.__name__))
                else:
                    component = current_view.__module__
                payload = {
                    'url': _request.base_url,
                    'component': component,
                    'action': _request.endpoint,
                    'params': {},
                    'session': filter_dict(dict(session), config.params_filters),
                    'cgi_data': {
                        k: v
                        for k, v in iteritems(_request.headers)
                        if k not in self.skip_headers
                    },
                    'context': context
                }

                # Add query params
                params = filter_dict(dict(_request.args), config.params_filters)
                params.update(filter_dict(dict(_request.form), config.params_filters))
                payload['params'] = params

                return payload

            return _wrapper

        payload.generic_request_payload = generic_request_payload_decorator(payload.generic_request_payload)
        logger.info('Monkey-patched generic_request_payload')

    def _handle_exception(self, sender, exception=None):
        """
        Actual code handling the exception and sending it to honeybadger if it's enabled.
        :param T sender: the object sending the exception event.
        :param Exception exception: the exception to handle.
        """
        self.handle_exception(exception=exception)



from __future__ import division, print_function, absolute_import

import logging

from honeybadger import payload

from celery import current_task
from celery.signals import task_failure, task_prerun, task_postrun
from .base import HoneybadgerExtension

logger = logging.getLogger(__name__)


class CeleryHoneybadgerFailureHandler(HoneybadgerExtension):

    def __init__(self):
        self.context_generators = {}
        self.report_exceptions = False

    def install(self, config={}, context_generators={}, report_exceptions=False):
        """
        Setup Celery - Honeybadger integration.
        :param dict[str, T] config: a configuration object to read config from.
        :param context_generators: Context generators
        :param bool report_exceptions: whether to automatically report exceptions on tasks or not.
        """
        self.initialize_honeybadger(config)
        self.context_generators = context_generators
        self.report_exceptions = report_exceptions
        task_prerun.connect(self.setup_context, weak=False)
        task_postrun.connect(self.reset_context, weak=False)
        if self.report_exceptions:
            task_failure.connect(self._failure_handler, weak=False)

        self._patch_generic_request_payload()
        logger.info('Registered Celery signal handlers')

    def _failure_handler(self, sender, task_id, exception, args, kwargs, traceback, einfo, **kw):
        """
        Handle failures.
        :param celery.Task sender: the task object sending the error.
        :param str task_id: the id of the task reporting the exception.
        :param Exception exception: the exception instance raised.
        :param list args: positional arguments the task was called with.
        :param dict kwargs: keyword arguments the task was called with.
        :param traceback: stack trace object.
        :param billiard.einfo.ExceptionInfo einfo: The billiard.einfo.ExceptionInfo instance.

        :param dict kw: any other arguments
        """
        self.handle_exception(exception=exception)

    def _patch_generic_request_payload(self):
        """
        Monkey-patches Honeybadger's generic_request_payload to add information from Celery task.
        """
        def generic_request_payload_decorator(original):
            def _wrapper(request, context, config):
                payload = {
                    'component': current_task.__module__,
                    'action': current_task.name,
                    'params': {
                        'args': list(current_task.request.args),
                        'kwargs': current_task.request.kwargs
                    },
                    'cgi_data': {
                        'task_id': current_task.request.id,
                        'retries': current_task.request.retries,
                        'max_retries': current_task.max_retries
                    },
                    'context': context
                }

                return payload

            return _wrapper

        payload.generic_request_payload = generic_request_payload_decorator(payload.generic_request_payload)
        logger.info('Monkey-patched generic_request_payload')

    def teardown(self):
        """
        Removes current failure handler
        """
        task_prerun.disconnect(self.setup_context)
        task_postrun.disconnect(self.reset_context)
        if self.report_exceptions:
            task_failure.disconnect(self._failure_handler)
        logger.info('Honeybadger Celery support uninstalled')



import logging
from honeybadger import honeybadger
from ._helpers import csv_to_list
from six import iteritems

logger = logging.getLogger(__name__)


class HoneybadgerExtension(object):
    """
    Base class for honeybadger extensions.
    """
    def __init__(self, context_generators={}, report_exceptions=False):
        """
        Initialize Honeybadger extension.
        :param dict context_generators: a dictionary with key the name of additional context property to add and value
        a callable that generates the actual value of the property.
        :param bool report_exceptions: whether to automatically report exceptions on requests or not.
        """
        self.context_generators = context_generators
        self.report_exception = report_exceptions

    def initialize_honeybadger(self, config):
        """
        Initializes honeybadger exception handler only if HONEYBADGER_API_KEY exists in config. If
        HONEYBADGER_ENVIRONMENT environment variable is set, honeybadger environment is also set.
        :param dict[str, T] config: the configuration object.
        """
        api_key = config.get('HONEYBADGER_API_KEY')
        # Initialize only if configured
        if api_key:
            logger.info('Configuring Honeybadger')
            honeybadger.configure(api_key=api_key,
                                  environment=config.get('HONEYBADGER_ENVIRONMENT', 'development'),
                                  params_filters=csv_to_list(config.get('HONEYBADGER_PARAMS_FILTERS',
                                                                        'password,password_confirmation,credit_card'))
                                  )
            logging.getLogger('honeybadger').addHandler(logging.StreamHandler())
            return True
        else:
            logger.info('No Honeybadger API KEY found, skipping configuration')
            return False

    def _generate_context(self):
        """
        Generate context for exception handling.
        :return: a dictionary with the context.
        :rtype: dict
        """
        context = {}
        for name, generator in iteritems(self.context_generators):
            context[name] = generator()

        return context

    def setup_context(self, *args, **kwargs):
        """
        Sets context for the request.
        :param T sender: the object sending the signal.
        :param extra: extra arguments passed by the signal.
        """
        honeybadger.set_context(**self._generate_context())

    def reset_context(self, *args, **kwargs):
        """
        Resets context when request is done.
        :param T sender: the object sending the signal
        :param Exception exc: exception caused during handling.
        :param extra: extras passed by the signal.
        """
        honeybadger.reset_context()

    def handle_exception(self, exception=None):
        """
        Actual code handling the exception and sending it to honeybadger if it's enabled.
        :param Exception exception: the exception to handle.
        """
        honeybadger.notify(exception)



from .celery import CeleryHoneybadgerFailureHandler
from .flask import HoneybadgerFlask


__version__ = '0.1.2'

celery_handler = CeleryHoneybadgerFailureHandler()

install_celery_handler = celery_handler.install
uninstall_celery_handler = celery_handler.teardown


__all__ = [
    'install_celery_handler',
    'uninstall_celery_handler',
    'HoneybadgerFlask'
]



"""
Honeybadger-Extensions
======================

|CircleCI|

**Honeybadger-Extensions** extend current `Honeybadger Python library`_ to better support `Celery`_ & `Flask`_.

It offers:

- Improved reporting, including details for component, action etc.
- Easier Honeybadger via Flask's or Celery's configuration object.
- (Optional) Automatic reporting of errors detected by Celery or Flask.


Features
--------

Honeybadger-Extensions provides the ``install_celery_handler()`` function which can be used
at any time which can be used to initialize both Honeybadger & allow improved Honeybadger reporting. It also offers the
``HoneybadgerFlask`` Flask extension that adds more information to Honeybadger logging, as well as automatic logging of
errors raised in the view functions.

Example 1: Initialize Celery
~~~~~~~~~~~~~~~~~~~~~~~~~~~~

In the following example, we will use the ``install_celery_handler`` to setup Honeybadger reporting.

.. code:: python

    from celery import Celery
    from honeybadger_extensions import install_celery_handler

    celery = Celery(__name__)
    celery.config_from_object({
        'HONEYBADGER_API_KEY': '<your key>',
        'HONEYBADGER_ENVIRONMENT': 'development'
    })

    install_celery_handler(config=celery.conf, report_exceptions=True)


Example 2: Initialize Flask
~~~~~~~~~~~~~~~~~~~~~~~~~~~

You can use HoneybadgerFlask extension to load Honeybadger configuration from Flask configuration object.

.. code:: python

    from flask import Flask, jsonify
    from honeybadger_extensions import HoneybadgerFlask

    app = Flask(__name__)
    app.config['HONEYBADGER_ENVIRONMENT'] = 'development'
    app.config['HONEYBADGER_API_KEY'] = '<your key>'
    app.config['HONEYBADGER_EXCLUDE_HEADERS'] = 'Authorization, Proxy-Authorization, X-Custom-Key'
    app.config['HONEYBADGER_PARAMS_FILTERS'] = 'password, secret, credit-card'

    # Setup Honeybadger and listen for exceptions
    HoneybadgerFlask(app, report_exceptions=True)

    @app.route('/')
    def index():
        a = int(request.args.get('a'))
        b = int(request.args.get('b'))

        logger.info('Dividing two numbers {} {}'.format(a, b))
        return jsonify({'result': a / b})

Installation
------------
The easiest way to install it is using ``pip`` from PyPI

.. code:: bash

    pip install Honeybadger-Extensions


License
-------

See the `LICENSE`_ file for license rights and limitations (MIT).

.. _Honeybadger Python Library: https://github.com/honeybadger-io/honeybadger-python
.. _Flask: http://flask.pocoo.org/
.. _Celery: http://www.celeryproject.org/
.. _LICENSE: https://github.com/Workable/honeybadger-extensions/blob/master/LICENSE.md
.. |CircleCI| image:: https://img.shields.io/circleci/project/github/Workable/honeybadger-extensions.svg
   :target: https://circleci.com/gh/Workable/honeybadger-extensions

"""
import re
import ast
from setuptools import setup

_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('honeybadger_extensions/__init__.py', 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')).group(1)))

test_requirements = [
    'nose',
    'mock==2.0.0',
    'coverage~=4.3.4'
]

setup(
    name='Honeybadger-Extensions',
    version=version,
    url='http://github.com/Workable/honeybadger-extensions',
    license='MIT',
    author='Ioannis Foukarakis, Konstantinos Paliouras',
    author_email='ioannis.foukarakis@gmail.com, squarious@gmail.com',
    description='Honeybadger extension that can log exceptions raised '
                'by Flask or Celery, adding more context than the original  '
                'honeybadger python library.',
    long_description=__doc__,
    maintainer="Ioannis Foukarakis",
    maintainer_email="ioannis.foukarakis@gmail.com",
    packages=[
        'honeybadger_extensions'
    ],
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=[
        'honeybadger==0.1.2',
        'blinker>=1.1',
        'Flask>=0.8',
        'celery~=4.1.0'
    ],
    tests_require=test_requirements,
    setup_requires=[
        "flake8",
        "nose"
    ],
    extras_require={
        'test': test_requirements,
        'celery': ["celery~=4.1.0"],
    },
    test_suite='nose.collector',
    classifiers=[
        'Environment :: Web Environment', 'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent', 'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ])
